﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Collections;
using System.IO;

namespace Lesson6
    {
    using Bentley.DgnPlatformNET;
    using Bentley.DgnPlatformNET.Elements;
    using Bentley.GeometryNET;
    using Bentley.CifNET.GeometryModel.SDK;
    using Bentley.CifNET.LinearGeometry;
    using Bentley.TerrainModelNET;
    public class AddSolidCommand : DgnElementSetTool
        {
        public AddSolidCommand() : base()
            {
            }

        protected override void OnRestartTool()
            {
            InstallNewInstance();
            }

        protected override void ExitTool()
            {
            //System.Windows.Forms.MessageBox.Show("ExitTool");
            base.ExitTool();
            }


        protected override void OnPostInstall()
            {
            //System.Windows.Forms.MessageBox.Show("OnPostInstall");
            }

        Alignment m_aligment = null;
        protected override bool OnDataButton(Bentley.DgnPlatformNET.DgnButtonEvent ev)
            {
            Bentley.DgnPlatformNET.HitPath hitPath = DoLocate(ev, true, 1);
            if (hitPath == null)
                return false;

            Element el = hitPath.GetCursorElement();
            if (el == null)
                return false;

            Alignment al = (el.ParentElement == null) ? Alignment.CreateFromElement(el) : Alignment.CreateFromElement(el.ParentElement);

            if (al == null)
                return false;
            m_aligment = al;

            DPoint3d meter_pt = GetMeterPoint(ev.Point);//ev.Point;
            ConsensusConnection connect = ConsensusConnection.Create(GetActiveModel());

            GeometricModel geomodel = connect.GetActive();

            foreach (TerrainSurface ti in geomodel.TerrainSurfaces)
                {

                if (ti.DTM == null)
                    {
                    continue;
                    }

                Bentley.TerrainModelNET.DTM dtmObject = ti.DTM;

                for (double dis = 0; dis <= m_aligment.LinearGeometry.Length; dis += 20)
                    {
                    LinearPoint pr_l = m_aligment.LinearGeometry.GetPointAtDistanceOffset(dis, -60);
                    LinearPoint pr_r = m_aligment.LinearGeometry.GetPointAtDistanceOffset(dis, 60);

                    DTMDrapedPoint dtm_ptl = dtmObject.DrapePoint(pr_l.Coordinates);

                    DgnModel threed = ti.DgnModel;
                    DPoint3d tmp = GetUORPoint(dtm_ptl.Coordinates);
                    BuildBoxSolidElement(threed, tmp, 20);

                    DTMDrapedPoint dtm_ptr = dtmObject.DrapePoint(pr_r.Coordinates);
                    tmp = GetUORPoint(dtm_ptr.Coordinates);
                    BuildBoxSolidElement(threed, tmp, 20);
                    }  
                }


            if (m_aligment != null)
                {
               
                BeginDynamics();
                return true;
                }
            return true;
            }

        protected override void OnDynamicFrame(DgnButtonEvent ev)
            {
            if (!DynamicsStarted)
                {
                return;
                }
            }
        public override Bentley.DgnPlatformNET.StatusInt OnElementModify(Bentley.DgnPlatformNET.Elements.Element element)
            {
            return Bentley.DgnPlatformNET.StatusInt.Error;
            }

        protected override bool OnResetButton(Bentley.DgnPlatformNET.DgnButtonEvent ev)
            {
            EndDynamics();            
            ExitTool();
            return true;
            }


        public static void InstallNewInstance()
            {
            AddSolidCommand tool = new AddSolidCommand();
            tool.InstallTool();
            }

        public static DPoint3d GetMeterPoint(DPoint3d uorpoint)
            {
            double u = UorPerMeter();

            return DPoint3d.FromXYZ(
                uorpoint.X / u,
                uorpoint.Y / u,
                uorpoint.Z / u);

            }

        public static DPoint3d GetUORPoint(DPoint3d meterpoint)
            {
            double u = UorPerMeter();

            return DPoint3d.FromXYZ(
                meterpoint.X * u,
                meterpoint.Y * u,
                meterpoint.Z * u);
            }
        public static double UorPerMeter()
            {
            ModelInfo modelInfo = Bentley.MstnPlatformNET.Session.Instance.GetActiveDgnModel().GetModelInfo();
            double uorsPerMeter = modelInfo.UorPerMeter;
            return uorsPerMeter;
            }
        public static DgnModel GetActiveModel()
            {
            return Bentley.MstnPlatformNET.Session.Instance.GetActiveDgnModel();
            }

        public bool processPointsFeatures(DTMDynamicFeatureInfo featureInfo, object oArg)
            {
            if (oArg != null)
                {
                ArrayList al = oArg as ArrayList;
                Bentley.CifNET.LinearGeometry.ClosedLineString cls = new Bentley.CifNET.LinearGeometry.ClosedLineString(featureInfo.FeaturePoints);
                al.Add(cls);
                }

            return true;
            }
        public static void BuildBoxSolidElement(DgnModel model, DPoint3d pos, double h)
            {
            DPoint3d baseOrigin = pos;
            DPoint3d topOrigin = pos;
            topOrigin.Z += h * UorPerMeter();

            double length = 2;
            DVector3d vectorX = new DVector3d(1, 0, 0);
            DVector3d vectorY = new DVector3d(0, 1, 0);
            double baseX = length * UorPerMeter();
            double baseY = length * UorPerMeter();
            double topX = length * UorPerMeter();
            double topY = length * UorPerMeter();
            bool capped = true;
            DgnBoxDetail box = new DgnBoxDetail(baseOrigin, topOrigin, vectorX, vectorY, baseX, baseY, topX, topY, capped);
            SolidPrimitive sp = SolidPrimitive.CreateDgnBox(box);


            Bentley.DgnPlatformNET.Elements.Element ee =
                Bentley.DgnPlatformNET.Elements.DraftingElementSchema.ToElement(model, sp, null);
            ee.AddToModel();
            return;
            }


        }
    }
